@@ -269,7 +269,7 @@ class Launch(FWSerializable, object):
             raise ValueError("Invalid launch state: {}".format(state))
 
         self.launch_dir = launch_dir
-        self.fworker = fworker
+        self.fworker = fworker if fworker else FWorker()
         self.host = host if host else get_my_host()
         self.ip = ip if ip else get_my_ip()
         self.action = action if action else None
@@ -387,7 +387,7 @@ class Launch(FWSerializable, object):
     @classmethod
     @recursive_deserialize
     def from_dict(cls, m_dict):
-        fworker = FWorker.from_dict(m_dict['fworker'])
+        fworker = FWorker.from_dict(m_dict['fworker']) if m_dict['fworker'] else None
         action = FWAction.from_dict(m_dict['action']) if m_dict.get(
             'action') else None
         return Launch(m_dict['state'], m_dict['launch_dir'], fworker,
@@ -630,7 +630,8 @@ class Workflow(FWSerializable):
 
             if new_fw.fw_id in leaf_ids:
                 if detour:
-                    self.links[new_fw.fw_id] = list(self.links[fw_id])  # add children of current FW to new FW
+                    self.links[new_fw.fw_id] = list(
+                        self.links[fw_id])  # add children of current FW to new FW
                 else:
                     self.links[new_fw.fw_id] = []
             else:
@@ -666,7 +667,7 @@ class Workflow(FWSerializable):
                          self.links.parent_links.get(fw_id, [])]
 
         completed_parent_states = ['COMPLETED']
-        if fw.spec.get('_allow_fizzled_parents', False):
+        if fw.spec.get('_allow_fizzled_parents'):
             completed_parent_states.append('FIZZLED')
 
         if len(parent_states) != 0 and not all(
@@ -689,6 +690,16 @@ class Workflow(FWSerializable):
                     if m_state == 'COMPLETED':
                         m_action = l.action
 
+            # This part is confusing and rare - report any FIZZLED parents if allow_fizzed
+            # allows us to handle FIZZLED jobs
+            if fw.spec.get('_allow_fizzled_parents'):
+                parent_fws = [self.id_fw[p].to_dict() for p in
+                                   self.links.parent_links.get(fw_id, []) if
+                                   self.id_fw[p].state == 'FIZZLED']
+                if len(parent_fws) > 0:
+                    fw.spec['_fizzled_parents'] = parent_fws
+                    updated_ids.add(fw_id)
+
         fw.state = m_state
 
         if m_state != prev_state:
@@ -697,9 +708,6 @@ class Workflow(FWSerializable):
             if m_state == 'COMPLETED':
                 updated_ids = updated_ids.union(self.apply_action(m_action,
                                                                   fw.fw_id))
-                # apply
-                # action
-                # on children
 
             # refresh all the children
             for child_id in self.links[fw_id]:
@@ -777,20 +785,23 @@ class Workflow(FWSerializable):
         nodes = sorted(m_dict['nodes'])
         m_dict['name--id'] = self.name + '--' + str(nodes[0])
         m_dict['launch_dirs'] = OrderedDict(
-            [(self.id_fw[x].name + '--' + str(x), [l.launch_dir for l in self.id_fw[x].launches])
+            [(self._str_fw(x), [l.launch_dir for l in self.id_fw[x].launches])
              for x in nodes])
         m_dict['states'] = OrderedDict(
-            [(self.id_fw[x].name + '--' + str(x), self.id_fw[x].state) for x in nodes])
-        m_dict['nodes'] = [self.id_fw[x].name for x in nodes]
+            [(self._str_fw(x), self.id_fw[x].state) for x in nodes])
+        m_dict['nodes'] = [self._str_fw(x) for x in nodes]
         m_dict['links'] = OrderedDict(
-            [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
+            [(self._str_fw(k), [self._str_fw(v) for v in a]) for k, a in
              m_dict['links'].iteritems()])
         m_dict['parent_links'] = OrderedDict(
-            [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
+            [(self._str_fw(k), [self._str_fw(v) for v in a]) for k, a in
              m_dict['parent_links'].iteritems()])
         m_dict['states_list'] = '-'.join([a[0:4] for a in m_dict['states'].values()])
         return m_dict
 
+    def _str_fw(self, fw_id):
+        return self.id_fw[int(fw_id)].name + '--' + str(fw_id)
+
     @classmethod
     def from_dict(cls, m_dict):
         # accept either a Workflow dict or a FireWork dict
@@ -804,5 +815,5 @@ class Workflow(FWSerializable):
             return Workflow.from_FireWork(FireWork.from_dict(m_dict))
 
     @classmethod
-    def from_FireWork(cls, fw):
-        return Workflow([fw], None)
\ No newline at end of file
+    def from_FireWork(cls, fw, name=None, metadata=None):
+        return Workflow([fw], None, name=name, metadata=metadata)
\ No newline at end of file
