@@ -36,7 +36,7 @@ class LaunchPad(FWSerializable):
     """
 
     def __init__(self, host='localhost', port=27017, name='fireworks', username=None, password=None,
-                 logdir=None, strm_lvl=None, user_indices=None):
+                 logdir=None, strm_lvl=None, user_indices=None, wf_user_indices=None):
         """
         
         :param host:
@@ -47,6 +47,7 @@ class LaunchPad(FWSerializable):
         :param logdir:
         :param strm_lvl:
         :param user_indices:
+        :param wf_user_indices:
         """
         self.host = host
         self.port = port
@@ -60,6 +61,7 @@ class LaunchPad(FWSerializable):
         self.m_logger = get_fw_logger('launchpad', l_dir=self.logdir, stream_level=self.strm_lvl)
 
         self.user_indices = user_indices if user_indices else []
+        self.wf_user_indices = wf_user_indices if wf_user_indices else []
 
         # get connection
         self.connection = MongoClient(host, port, j=True)
@@ -78,7 +80,7 @@ class LaunchPad(FWSerializable):
         """
         d = {'host': self.host, 'port': self.port, 'name': self.name, 'username': self.username,
              'password': self.password, 'logdir': self.logdir, 'strm_lvl': self.strm_lvl,
-             'user_indices': self.user_indices}
+             'user_indices': self.user_indices, 'wf_user_indices': self.wf_user_indices}
         return d
 
     @classmethod
@@ -86,8 +88,9 @@ class LaunchPad(FWSerializable):
         logdir = d.get('logdir', None)
         strm_lvl = d.get('strm_lvl', None)
         user_indices = d.get('user_indices', [])
+        wf_user_indices = d.get('wf_user_indices', [])
         return LaunchPad(d['host'], d['port'], d['name'], d['username'], d['password'], logdir,
-                         strm_lvl, user_indices)
+                         strm_lvl, user_indices, wf_user_indices)
 
     def reset(self, password, require_password=True):
         """
@@ -135,7 +138,7 @@ class LaunchPad(FWSerializable):
             self.m_logger.debug('Sleeping for {} secs...'.format(maintain_interval))
             time.sleep(maintain_interval)
 
-    def add_wf(self, wf):
+    def add_wf(self, wf, reassign_all=True):
         """
 
         :param wf: a Workflow object.
@@ -145,7 +148,7 @@ class LaunchPad(FWSerializable):
             wf = Workflow.from_FireWork(wf)
 
         # insert the FireWorks and get back mapping of old to new ids
-        old_new = self._upsert_fws(wf.id_fw.values(), reassign_all=True)
+        old_new = self._upsert_fws(wf.id_fw.values(), reassign_all=reassign_all)
 
         # update the Workflow with the new ids
         wf._reassign_ids(old_new)
@@ -264,6 +267,9 @@ class LaunchPad(FWSerializable):
         for idx in self.user_indices:
             self.fireworks.ensure_index(idx)
 
+        for idx in self.wf_user_indices:
+            self.workflows.ensure_index(idx)
+
         self.m_logger.debug('Compacting database...')
         try:
             self.db.command({'compact': 'fireworks'})
@@ -551,8 +557,9 @@ class LaunchPad(FWSerializable):
         updated_fws = [wf.id_fw[fid] for fid in updated_ids]
         old_new = self._upsert_fws(updated_fws)
         wf._reassign_ids(old_new)
-        # redo the links
-        self.workflows.find_and_modify({'nodes': wf.fws[0].fw_id}, wf.to_db_dict())
+        # redo the links - note that if you don't search across all keys, you can get errors
+        self.workflows.find_and_modify({'nodes': {'$in': wf.id_fw.keys()}}, wf.to_db_dict())
+
 
     def _steal_launches(self, thief_fw):
         stolen = False
@@ -585,16 +592,9 @@ class LaunchPad(FWSerializable):
 
     def _upsert_launch(self, m_launch):
         # Do a confirmed write of Launch
+        self.launches.find_and_modify({'launch_id': m_launch.launch_id}, m_launch.to_db_dict(), upsert=True)
 
-        l_id = m_launch.launch_id
-        old_launch = self.launches.find_and_modify({'launch_id': l_id}, m_launch.to_db_dict(),
-                                                   new=False, fields={"state": True}, upsert=True)
-
-        if old_launch and old_launch['state'] == 'COMPLETED':
-            self.m_logger.error(
-                'launch id: {} tried to overwrite COMPLETED state! Marking as FIZZLED'.format(l_id))
-            self.mark_fizzled(l_id)
-
+        """
         # confirm write
         # I can't believe this is actually necessary (and yes, it appears to be necessary)
         nloops = 0
@@ -614,4 +614,5 @@ class LaunchPad(FWSerializable):
             if nloops % 40 == 0:
                 self.m_logger.warning('Fixing a lost write of launch_id: {}'.format(l_id))
                 self.launches.find_and_modify({'launch_id': l_id}, m_launch.to_db_dict())
-            time.sleep(4)
\ No newline at end of file
+            time.sleep(4)
+        """
\ No newline at end of file
